@@ -376,7 +376,7 @@ pty_input_putback (const char *str, size_t len)
 /* pty_read()
  *
  * Read errors EWOULDBLOCK, EAGAIN, and EIO are
- * tweeked into reporting zero bytes input.
+ * tweaked into reporting zero bytes input.
  * In particular, EIO is known to appear when
  * reading off the master side, before having
  * an active slave side.
@@ -726,7 +726,7 @@ getterminaltype (char *uname, size_t len)
    * Distinguish the available modes by level:
    *
    *   off:                     Authentication is forbidden.
-   *   none:                    Volontary authentication.
+   *   none:                    Voluntary authentication.
    *   user, valid, other:      Mandatory authentication only.
    */
   if (auth_level < 0)
@@ -1685,6 +1685,17 @@ static void _expand_cond (struct line_expander *exp);
 static void _skip_block (struct line_expander *exp);
 static void _expand_block (struct line_expander *exp);
 
+static char *
+sanitize (const char *u)
+{
+  /* Ignore values starting with '-' or containing shell metachars, as
+     they can cause trouble.  */
+  if (u && *u != '-' && !u[strcspn (u, "\t\n !\"#$&'()*;<=>?[\\^`{|}~")])
+    return u;
+  else
+    return "";
+}
+
 /* Expand a variable referenced by its short one-symbol name.
    Input: exp->cp points to the variable name.
    FIXME: not implemented */
@@ -1711,13 +1722,13 @@ _var_short_name (struct line_expander *exp)
       return xstrdup (timebuf);
 
     case 'h':
-      return xstrdup (remote_hostname);
+      return xstrdup (sanitize (remote_hostname));
 
     case 'l':
-      return xstrdup (local_hostname);
+      return xstrdup (sanitize (local_hostname));
 
     case 'L':
-      return xstrdup (line);
+      return xstrdup (sanitize (line));
 
     case 't':
       q = strchr (line + 1, '/');
@@ -1725,16 +1736,16 @@ _var_short_name (struct line_expander *exp)
 	q++;
       else
 	q = line;
-      return xstrdup (q);
+      return xstrdup (sanitize (q));
 
     case 'T':
-      return terminaltype ? xstrdup (terminaltype) : NULL;
+      return terminaltype ? xstrdup (sanitize (terminaltype)) : NULL;
 
     case 'u':
-      return user_name ? xstrdup (user_name) : NULL;
+      return user_name ? xstrdup (sanitize (user_name)) : NULL;
 
     case 'U':
-      return getenv ("USER") ? xstrdup (getenv ("USER")) : xstrdup ("");
+      return xstrdup (sanitize (getenv ("USER")));
 
     default:
       exp->state = EXP_STATE_ERROR;
